@@ -1,12 +1,15 @@
 #!/usr/bin/env python
 # coding: utf-8
-import numpy as np
-import xarray as xr
+import errno
+import os
 from pathlib import Path
-import xmltodict
+
+import numpy as np
 import pandas as pd
-from munch import munchify, Munch
 import scipy.io as sio
+import xarray as xr
+import xmltodict
+from munch import Munch, munchify
 
 from .helpers import mtlb2datetime
 
@@ -51,19 +54,34 @@ class CTD(object):
         self.parse_hex()
         self.physicalunits()
 
-    def _detect_missing_word(self):
-        """Sometimes there is no SPAR sensor, in this case one hex word may be missing.
-        It lives on voltage channel 9.
+    def _detect_missing_words(self):
+        """
+        Determine location of data entries in hex file line.
+
+        Sometimes there is no SPAR sensor, in this case one hex word may
+        be missing. It lives on voltage channel 9.
+
+        Locating the right data entries also depends on the number of
+        bytes written per scan. We read this information from the
+        header.
+
+        For details, see p.67 in manual-11pV2_018.pdf
         """
         if ~hasattr(self, "cfgp"):
             self.read_xml_config()
         if (
-            14 not in self.cfgp.loc["@index"].values
+            "14" not in self.cfgp.loc["@index"].values
             and "SPAR_Sensor" not in self.cfgp.loc["@index"].keys()
         ):
             self._hexoffset = -6
         else:
             self._hexoffset = 0
+        if "14" in self.cfgp.loc["@index"].values and self._bytes_per_scan == 48:
+            self._extra_hexoffset = 8
+        elif "14" not in self.cfgp.loc["@index"].values and self._bytes_per_scan == 45:
+            self._extra_hexoffset = 8
+        else:
+            self._extra_hexoffset = 0
 
     def parse_hex(self):
         # Generate data structure for converted data: 5 freq, 8 voltage channels
@@ -79,7 +97,6 @@ class CTD(object):
         tmp["ctdstatus"] = []
         tmp["lon"] = []
         tmp["lat"] = []
-        self._detect_missing_word()
 
         out = dict(header="")
 
@@ -90,10 +107,14 @@ class CTD(object):
                     pass
                 elif l[0] == "*":
                     out["header"] += l
-                    if l[0:5] == "*    ":
+                    # if l[0:5] == "*    ":
+                    # i = l.find("=")
+                    if l[0:17] == "* Number of Bytes":
                         i = l.find("=")
+                        self._bytes_per_scan = int(l[i + 2 :])
                 else:
                     break
+            self._detect_missing_words()
             # read data
             for l in fin:
                 # parse frequency channels
@@ -106,31 +127,74 @@ class CTD(object):
                     v1, v2 = self._hexword2volt(l[slice((i + 5) * 6, (i + 5) * 6 + 6)])
                     tmp["v{}".format(i * 2)].append(v1)
                     tmp["v{}".format(i * 2 + 1)].append(v2)
-                # parse other
-                tmp["modcount"].append(
-                    int(l[78 + self._hexoffset : 80 + self._hexoffset], 16)
-                )
-                tmp["time"].append(
-                    int(
-                        l[86 + self._hexoffset : 88 + self._hexoffset]
-                        + l[84 + self._hexoffset : 86 + self._hexoffset]
-                        + l[82 + self._hexoffset : 84 + self._hexoffset]
-                        + l[80 + self._hexoffset : 82 + self._hexoffset],
-                        16,
-                    )
-                )
-                pst, ctdstatus = self._hexword2pstat(
-                    l[slice(74 + self._hexoffset, 78 + self._hexoffset)]
-                )
-                tmp["pst"].append(pst)
-                tmp["ctdstatus"].append(ctdstatus)
-                if self._hexoffset != 0:
+                # parse other channels
+                # spar
+                if self._hexoffset == 0:
                     tmp["spar"].append(self._hexword2spar(l[slice(57, 60)]))
-                b, lat, lon = self._hexword2lonlat(
+                # gps
+                lat, lon = self._hexword2lonlat(
                     l[60 + self._hexoffset : 74 + self._hexoffset]
                 )
                 tmp["lon"].append(lon)
                 tmp["lat"].append(lat)
+                # pressure sensor temperature and ctd status
+                pst, ctdstatus = self._hexword2pstat(
+                    l[
+                        slice(
+                            74 + self._hexoffset + self._extra_hexoffset,
+                            78 + self._hexoffset + self._extra_hexoffset,
+                        )
+                    ]
+                )
+                tmp["pst"].append(pst)
+                tmp["ctdstatus"].append(ctdstatus)
+                # modcount
+                tmp["modcount"].append(
+                    int(
+                        l[
+                            78
+                            + self._hexoffset
+                            + self._extra_hexoffset : 80
+                            + self._hexoffset
+                            + self._extra_hexoffset
+                        ],
+                        16,
+                    )
+                )
+                # time
+                tmp["time"].append(
+                    int(
+                        l[
+                            86
+                            + self._hexoffset
+                            + self._extra_hexoffset : 88
+                            + self._hexoffset
+                            + self._extra_hexoffset
+                        ]
+                        + l[
+                            84
+                            + self._hexoffset
+                            + self._extra_hexoffset : 86
+                            + self._hexoffset
+                            + self._extra_hexoffset
+                        ]
+                        + l[
+                            82
+                            + self._hexoffset
+                            + self._extra_hexoffset : 84
+                            + self._hexoffset
+                            + self._extra_hexoffset
+                        ]
+                        + l[
+                            80
+                            + self._hexoffset
+                            + self._extra_hexoffset : 82
+                            + self._hexoffset
+                            + self._extra_hexoffset
+                        ],
+                        16,
+                    )
+                )
 
             # generate output array
             # frequency variables are always there
@@ -152,6 +216,8 @@ class CTD(object):
             out["ctdstatus"] = tmp["ctdstatus"]
             out["modcount"] = np.array(tmp["modcount"])
             out["time"] = np.array(tmp["time"])
+            if "spar" in tmp:
+                out["spar"] = np.array(tmp["spar"])
             self.dataraw = munchify(out)
 
     def _hexword2freq(self, hex_str):
@@ -227,7 +293,7 @@ class CTD(object):
             longitude and latitude
         """
         b = format(int(hex_str[12:14], 16), "08b")
-        newpos = int(b[7])
+        # newpos = int(b[7])
         lonneg = int(b[1])
         latneg = int(b[0])
         lat = (
@@ -248,7 +314,7 @@ class CTD(object):
             )
             / 5e4
         )
-        return newpos, lat, lon
+        return lat, lon
 
     def _hexword2spar(self, hex_str):
         """
@@ -306,15 +372,24 @@ class CTD(object):
         Config file needs to be in the same directory as the hex file."""
         pp = Path(self.filename)
         name = pp.stem
+        # try upper case filename
         xmlfile = name.upper() + ".XMLCON"
         p = pp.parent
         self.xmlfile = p.joinpath(xmlfile)
+        # use os.listdir to find the actual case of the filename if the upper
+        # case did not work.
+        if self.xmlfile.name not in os.listdir(os.path.dirname(self.xmlfile)):
+            xmlfile = name.lower() + ".XMLCON"
+            self.xmlfile = p.joinpath(xmlfile)
 
     def read_xml_config(self):
         """Read xml config file."""
         self._find_xmlconfig()
-        with open(self.xmlfile) as fd:
-            tmp = xmltodict.parse(fd.read())
+        try:
+            with open(self.xmlfile) as fd:
+                tmp = xmltodict.parse(fd.read())
+        except OSError as e:
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), e.filename)
         tmp = tmp["SBE_InstrumentConfiguration"]
         tmp = tmp["Instrument"]
         sa = tmp["SensorArray"]["Sensor"]
@@ -359,16 +434,19 @@ class CTD(object):
             # convert any None items to 'N/A'
             for ki, v in self.cfgp[k].cal.items():
                 if v is None:
-                    self.cfgp[k].cal[ki] = 'N/A'
+                    self.cfgp[k].cal[ki] = "N/A"
 
     def physicalunits(self):
         # pressure
-        self._p_atm = 10.1353
+        self._p_atm = 10.1353  # why not 10.1325 dbar?
         self.data = Munch()
         self.data.lon = self.dataraw.lon
         self.data.lat = self.dataraw.lat
-        self.data.p = self._freq2pressure(
-            self.dataraw.p, self.dataraw.pst, self.cfgp.PressureSensor.cal
+        self.data.p = (
+            self._freq2pressure(
+                self.dataraw.p, self.dataraw.pst, self.cfgp.PressureSensor.cal
+            )
+            - self._p_atm
         )
         self.data.t1 = self._freq2temp(
             self.dataraw.t1, self.cfgp.TemperatureSensor1.cal
@@ -421,7 +499,7 @@ class CTD(object):
         return pres
 
     def _freq2temp(self, freq, tcal):
-        """Calculate  temperature given frequency and 
+        """Calculate  temperature given frequency and
         temperature calibration structure tcal
         D. Rudnick 01/06/05"""
 
