@@ -1,9 +1,12 @@
 #!/usr/bin/env python
 # coding: utf-8
+import gsw
+import matplotlib as mpl
 import numpy as np
 import xarray as xr
-from scipy import signal, optimize, fft, stats
-import matplotlib as mpl
+from scipy import fft, optimize, signal, stats
+
+from . import calcs, helpers
 
 # We are running into trouble if the default backend is not installed
 # in the current python environment. Try to import pyplot the default
@@ -13,8 +16,6 @@ try:
 except ImportError:
     mpl.use("agg")
     import matplotlib.pyplot as plt
-import gsw
-from . import helpers, calcs
 
 
 def run_all(data):
@@ -435,7 +436,7 @@ def ctd_correction2(data, tcfit, plot_spectra=None, plot_path=None):
 
     # Corrected Fourier transforms of temperature.
     Ad["t1"] = Ad["t1"] * ((H1 * LP) * np.ones((2 * m - 1, 1)))
-    Ad["t2"] = Ad["t2"] * ((H1 * LP) * np.ones((2 * m - 1, 1)))
+    Ad["t2"] = Ad["t2"] * ((H2 * LP) * np.ones((2 * m - 1, 1)))
 
     # LP filter remaining variables
     vars2 = [
@@ -483,7 +484,7 @@ def ctd_correction2(data, tcfit, plot_spectra=None, plot_path=None):
     t2 = Adi["t2"][int(N / 4) : -int(N / 4)]
     c1 = Adi["c1"][int(N / 4) : -int(N / 4)]
     c2 = Adi["c2"][int(N / 4) : -int(N / 4)]
-    p = Adi["p"][int(N / 4) : -int(N / 4)]
+    # p = Adi["p"][int(N / 4) : -int(N / 4)]
 
     m = (i2 - N) / N  # number of segments = dof/2
     m = np.floor(m).astype("int64")
@@ -640,7 +641,6 @@ def ctd_rmloops(data, wthresh=0.1):
     """
     tsmooth = 0.25  # seconds
     fs = 24  # Hz
-    pn = data.p.size
     w = calcs.wsink(data.p.data, tsmooth, fs)  # down/up +/-ve
     iloop = np.array([])
 
@@ -653,36 +653,38 @@ def ctd_rmloops(data, wthresh=0.1):
         flp = np.squeeze(np.argwhere(w < wthresh))
         if flp.size >= 1:
             ia, ib, ilen = helpers.findsegments(flp)
-            nlp = ia.size
+            if ia[0] == ib[0] == 0:
+                ia = np.delete(ia, 0)
+                ib = np.delete(ib, 0)
             for start, stop in zip(ia, ib):
                 pmi = np.argmax(data.p[:stop]).data
                 pm = data.p[pmi].data
                 tmp = np.squeeze(np.where(data.p[pmi:] < pm))
                 iloop = np.append(iloop, pmi + 1)
                 iloop = np.append(iloop, pmi + tmp - 1)
+                iloop = np.unique(iloop)
             iloop = iloop.astype("int64")
         else:
-            nlp = np.array([])
             iloop = np.array([])
     else:
         # upcast
         flp = np.squeeze(np.argwhere(w > -wthresh))
         if flp.size >= 1:
             ia, ib, ilen = helpers.findsegments(flp)
-            nlp = ia.size
+            if ia[0] == ib[0] == 0:
+                ia = np.delete(ia, 0)
+                ib = np.delete(ib, 0)
             for start, stop in zip(ia, ib):
                 pmi = np.argmax(data.p[:stop]).data
                 pm = data.p[pmi].data
                 tmp = np.squeeze(np.where(data.p[:pmi] < pm))
                 iloop = np.append(iloop, pmi)
                 iloop = np.append(iloop, pmi - tmp)
+                iloop = np.unique(iloop)
             iloop = iloop.astype("int64")
         else:
-            nlp = np.array([])
             iloop = np.array([])
 
-    iloop2 = np.unique(iloop)
-
     # get data variable names
     varnames = [k for k, v in data.data_vars.items()]
     for rmitem in ["lon", "lat", "z", "depth"]:
@@ -690,7 +692,7 @@ def ctd_rmloops(data, wthresh=0.1):
             varnames.remove(rmitem)
     # set loops to NaN
     for varn in varnames:
-        data[varn][iloop2] = np.nan
+        data[varn][iloop] = np.nan
 
     return data
 
